/* $Id: base64.c,v 1.1 2016/11/02 05:10:53 rlr Exp rlr $ */


/*
 * Copyright (c) 2016 Ross L Richardson <rueda@rlr.id.au>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#include <netinet/in.h>

#include <resolv.h>
#include <string.h>

#include "base64.h"


/* Base64 constants */
static const char	*b64_chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                                     "abcdefghijklmnopqrstuvwxyz0123456789+/";
static const char	b64_pad = '=';


int
base64_encode(const void *src, size_t src_sz, char *dst, size_t dst_sz)
{
	return b64_ntop(src, src_sz, dst, dst_sz);
}


int
base64_decode(const char *src, void *dst, size_t dst_sz)
{
	return b64_pton(src, dst, dst_sz);
}


int
base64_is_well_formed(const char *e)
{
	size_t	 len, bc;
	int	 res;

	res = 0;  /* until proven otherwise */

	len = strlen(e);
	if (len > 0 && (len % 4) == 0) {
		/* reasonable length, so check content */
		bc = strspn(e, b64_chars);

		res = bc == len || (bc == len-1 && e[bc] == b64_pad)
		    || (bc == len-2 && e[bc] == b64_pad && e[bc+1] == b64_pad);
	}

	return res;
}


size_t
base64_decoded_length(const char *e)
{
	size_t	 len_bin;
	size_t	 len_txt;

	len_txt = strlen(e);
	len_bin = len_txt / 4 * 3;

	if (len_bin > 0) {
		while (e[len_txt - 1] == b64_pad) {
			len_bin--;
			len_txt--;
		}
	}

	return len_bin;
}

