/* $Id: conf_parse.h,v 1.4 2017/08/05 11:55:03 rlr Exp $ */

/*
 * Copyright (c) 2017 Ross Richardson <ross@rlr.id.au>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _CONF_PARSE_H_
#define _CONF_PARSE_H_


#include <stddef.h>


/*
 * Error codes.
 * Postive values indicate line number of parsing or callback error.
 */
#define CP_NO_ERR        0
#define CP_OPEN_ERR     -1
#define CP_STAT_ERR     -2
#define CP_TOO_BIG_ERR  -3
#define CP_MMAP_ERR     -4


/*
 * int
 * conf_parse_buf(const char *buf, size_t len,
 *     int (*cb)(const char *n, size_t nlen, const char *v, size_t vlen,
 *     void *p), void *a);
 *
 * conf_parse_buf is a simple parsing function for parsing text consisting of
 * lines containing name/value pairs.
 *
 * Valid lines are one of the following:
 * - empty
 * - comments, which start with '#'
 * - name (which must start at start of line) +/- value, separated by one or
 *   more space and/or tab characters
 *
 * buf contains arbitrary text of length len.  Parsing continues until EOF
 * or until an error is found.  cb is called for each line successfully
 * parsed.  (If cb is NULL, the effect is undefined.)  cb is passed pointers
 * and lengths for the name and value, and arbitrary a.  cb must return 0
 * on success, non-zero on error.
 *
 * If a line contains just a name (with no following space/tab character),
 * cb is passed vlen 0 and v NULL; if a line contains name and following
 * space/tab, cb is passed vlen 0 and non-NULL v.
 *
 * conf_parse_buf returns CP_NO_ERR (zero) in the absence of error, or the
 * line number on which a parsing error occurred, or for which the callback
 * function reported failure.
 */
int
conf_parse_buf(const char *, size_t,
    int (*)(const char *, size_t, const char *, size_t, void *), void *);


/*
 * int
 * conf_parse_file(const char *,
 *     int (*)(const char *, size_t, const char*, size_t, void *), void *);
 *
 * Wrapper for conf_parse_buf which uses data from file filepath.  Negative
 * return values (see defines) indicate failed system calls prior to parsing.
 *
 * On OpenBSD, requires pledge(2) promise "rpath".
 */
int
conf_parse_file(const char *,
    int (*)(const char *, size_t, const char*, size_t, void *), void *);


#endif /* !_CONF_PARSE_H_ */

