/* $Id: fwobac.h,v 1.86 2017/09/02 11:01:58 rlr Exp $ */

/*
 * Copyright (c) 2017 Ross Richardson <ross@rlr.id.au>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _FWOBAC_H_
#define _FWOBAC_H_

/* Configuration file path */
#ifndef CONF_FILE
#define CONF_FILE  "/etc/fwobac.conf"
#endif /* CONF_FILE */

#ifdef FEATURE_THEMES
/* Theme list configuration entry is a comma-separated list. */
#define THEME_LIST_SEP  ','
#endif /* FEATURE_THEMES */

/* Group rights configuration entry is of form: grp0:spec,grp1:spec */
#define GROUP_RIGHTS_SPEC_SEP  '='
#define GROUP_RIGHTS_LIST_SEP  ';'

/* Names of pre-defined groups */
#define PSEUDO_GROUP_ANON  "anon"  /* pseudo-group for not-logged-in */
#define GROUP_PART_REG     "none"  /* registration as yet unconfirmed */
#define GROUP_USER         "user"  /* general (fully registered) user */

/* Separation of alternatives in conditional template macros */
#define T_MACRO_ALT_BRA  ':'
#define T_MACRO_ALT_KET  ':'  /* T_MACRO_ALT_BRA c T_MACRO_ALT_KET */

/* Format for authentication cookie */
#define AUTH_COOKIE_FMT  KN_AUTH_COOKIE "=%s; path=/; domain=%s; Secure;" \
    " HttpOnly; SameSite=Strict"

/* Formats for names of generated files */
#define FMT_LAST_ART_CH  "%s/last_art_ch.%d"  /* Y10K problem */
#define FMT_TEMPLATE_ARCHIVE_INDEX  "%s/archive_index.%d"

#ifdef UNDEADLY_TRANSITION
#define FMT_URL_ERRATA  "%s://%s/errata/errata.rss"
#endif /* UNDEADLY_TRANSITION */

#ifdef FEATURE_SEARCH
/* Search engine */
#define FMT_SE  "%s://duckduckgo.com/lite/?q=site:%s+%s" /* Search engine. */
#endif /* FEATURE_SEARCH */

/* Number of articles */
#define ARTICLES_FRONT       9
#define ARTICLES_OLDER_LIST  9

/* strftime(3) formats */
#define DEFAULT_TIME_FMT   "%Y-%m-%d %H:%M:%S %Z"
#define RFC_822_FMT        "%a, %d %b %Y %H:%M:%S GMT"
#define ISO_8601_FMT       "%Y-%m-%dT%H:%M:%SZ"
#define TIMESTAMP_FMT      "%Y%m%d%H%M%S"
#define TIMESTAMP_LEN      14  /* yyyymmddHHMMSS */

/* Formats for duration strings */
#define FMT_DS_DHM  "%ldd%d:%2.2d"
#define FMT_DS_DM   "%ldd%dm"
#define FMT_DS_HM   "%d:%2.2d"
#define FMT_DS_M    "%dm"

/* Number of items for RSS feed */
#define RSS_ITEMS_MIN       1
#define RSS_ITEMS_MAX      16
#define RSS_ITEMS_DEFAULT   8

/* Length of generated CAPTCHA string */
#define CAPTCHA_LEN         8

/* For injecting into HTML <a> elements from untrusted sources */
#define ATTR_REL_NO_FOLLOW  "rel=\"nofollow\""

/* UTC timezone name */
#define UTC_NAME  "UTC"

/* Fixed strings (HTML) */
#define FS_H_ANON_NAME   "Anonymous Coward"
#define FS_H_AT          "@"
#define FS_H_COMMENTS_0  "No comments"
#define FS_H_COMMENTS_1  "1 comment"
#define FS_H_INT_ERR     "<p>[Internal Error]</p>"
#ifdef FEATURE_A_PREV_NEXT_LINKS
#define FS_H_NEXT        "&gt;"
#define FS_H_PREV        "&lt;"
#define FS_H_SEP         "|"
#define FS_H_UP_POST     "^|"
#define FS_H_UP_PRE      "|^"
#endif /* FEATURE_A_PREV_NEXT_LINKS */
#define FS_H_UNKNOWN     "?"

/* Fixed string formats (HTML). */
#define FMT_H_COMMENTS_N  "%u comments"
#define FMT_H_COMMENTS_T  " (%s ago)"

#ifdef FEATURE_COMMENT_SUBJECT
/* Fixed strings (text) */
#define FS_T_NO_SUBJ     "No Subject Given"
#endif /* FEATURE_COMMENT_SUBJECT */

/* Set Nth Bit */
#define SNB(n)	(1ULL << n)

/*
 * Key (field [URL field=val] and cookie) names
 *
 * If modifying these, remember to maintain the entries in keys[] (and
 * key_desc[]) in fwobac.c in alphabetical order (for binary searching).
 */
#define KN_ACTION       "action"
#define KN_ADDR_AUTHOR  "addr_author"
#define KN_ADDR_EDITOR  "addr_editor"
#define KN_ASK          "ask"
#define KN_AUTH_COOKIE  "auth"
#define KN_CAPTCHA      "captcha"
#define KN_CAPTCHA_CHK  "captcha_check"
#define KN_CONTENT0     "content0"
#define KN_CONTENT1     "content1"
#ifdef DEBUG
#define KN_DEBUG        "debug"
#endif /* DEBUG */
#define KN_DEPT         "dept"
#define KN_EMAIL        "email"
#define KN_H            "h"
#define KN_HREF         "href"
#define KN_ID           "id"
#define KN_ID_AUTHOR    "id_author"
#define KN_ID_EDITOR    "id_editor"
#define KN_ITEMS        "items"
#define KN_NAME         "name"
#define KN_PID          "pid"
#define KN_POLL         "poll"
#define KN_PREVIEW      "preview"
#define KN_PUBDATE      "pubdate"
#define KN_PW           "pw"
#define KN_PWCONFIRM    "pwconfirm"
#define KN_Q            "q"
#define KN_SID          "sid"
#define KN_SUBJECT      "subject"
#ifdef FEATURE_THEMES
#define KN_THEME        "theme"
#endif /* FEATURE_THEMES */
#ifdef FEATURE_THRES
#define KN_THRES        "thres"
#endif /* FEATURE_THRES */
#define KN_TOPIC        "topic"
#define KN_TS_AUTHORED  "ts_authored"
#define KN_TYPE         "type"
#if defined(FEATURE_MODERATE) || defined (FEATURE_VOTE)
#define KN_VAL          "val"
#endif /* defined(FEATURE_MODERATE) || defined (FEATURE_VOTE) */
#define KN_YEAR         "y"

/* Key descriptions (HTML) */
#define KD_ACTION       "action"
#define KD_ADDR_AUTHOR  "author IP address"
#define KD_ADDR_EDITOR  "editor IP address"
#define KD_ASK          "confirmation setting"
#define KD_AUTH_COOKIE  "authorisation cookie"
#define KD_CAPTCHA      "<abbr>CAPTCHA</abbr>"
#define KD_CAPTCHA_CHK  "<abbr>CAPTCHA</abbr> check"
#define KD_CONTENT0     "article/comment content"
#define KD_CONTENT1     "additional article content"
#ifdef DEBUG
#define KD_DEBUG        "debug setting"
#endif /* DEBUG */
#define KD_DEPT         "department"
#define KD_EMAIL        "email address"
#define KD_H            "<abbr>HMAC</abbr>"
#define KD_HREF         "<abbr>URL</abbr>"
#define KD_ID           "<abbr>ID</abbr>"
#define KD_ID_AUTHOR    "author <abbr>ID</abbr>"
#define KD_ID_EDITOR    "editor <abbr>ID</abbr>"
#define KD_ITEMS        "number of items"
#define KD_NAME         "name"
#define KD_PID          "comment <abbr>ID</abbr>"
#define KD_POLL         "poll setting"
#define KD_PREVIEW      "preview setting"
#define KD_PUBDATE      "publication date"
#define KD_PW           "password"
#define KD_PWCONFIRM    "confirmation password"
#define KD_Q            "query"
#define KD_SID          "article <abbr>ID</abbr>"
#define KD_SUBJECT      "subject"
#ifdef FEATURE_THEMES
#define KD_THEME        "theme"
#endif /* FEATURE_THEMES */
#ifdef FEATURE_THRES
#define KD_THRES        "threshold"
#endif /* FEATURE_THRES */
#define KD_TOPIC        "topic"
#define KD_TS_AUTHORED  "authored timestamp"
#define KD_TYPE         "type"
#if defined(FEATURE_MODERATE) || defined (FEATURE_VOTE)
#define KD_VAL          "value"
#endif /* defined(FEATURE_MODERATE) || defined (FEATURE_VOTE) */
#define KD_YEAR         "year"

/* Keys: fields and cookie(s) */
enum key {
	K_ACTION,
	K_ADDR_AUTHOR,
	K_ADDR_EDITOR,
	K_ASK,
	K_AUTH_COOKIE,
	K_CAPTCHA,
	K_CAPTCHA_CHK,
	K_CONTENT0,
	K_CONTENT1,
#ifdef DEBUG
	K_DEBUG,
#endif /* DEBUG */
	K_DEPT,
	K_EMAIL,
	K_H,
	K_HREF,
	K_ID,
	K_ID_AUTHOR,
	K_ID_EDITOR,
	K_ITEMS,
	K_NAME,
	K_PID,
	K_POLL,
	K_PREVIEW,
	K_PUBDATE,
	K_PW,
	K_PWCONFIRM,
	K_Q,
	K_SID,
	K_SUBJECT,
#ifdef FEATURE_THEMES
	K_THEME,
#endif /* FEATURE_THEMES */
#ifdef FEATURE_THRES
	K_THRES,
#endif /* FEATURE_THRES */
	K_TOPIC,
	K_TS_AUTHORED,
	K_TYPE,
#if defined(FEATURE_MODERATE) || defined (FEATURE_VOTE)
	K_VAL,
#endif /* defined(FEATURE_MODERATE) || defined (FEATURE_VOTE) */
	K_YEAR
};


/* For field K_ACTION */
enum action {
	A_ABOUT,
	A_ARCHIVE,
	A_ARTICLE,
	A_AUTH,
	A_BLOCK_ADDR,
	A_CHANGEPW,
	A_CLOSE_REPORT,
	A_CLOSE_COMMENTS,
	A_CONFIRM,
	A_DEAUTH,
	A_DELETE_A,
	A_DELETE_C,
	A_DELETE_S,
	A_EDIT_A,
	A_EDIT_S,
	A_EDITPOST_A,
	A_EDITPOST_S,
#ifdef UNDEADLY_TRANSITION
	A_ERRATA,
#endif /* UNDEADLY_TRANSITION */
	A_FRONT,
	A_LOCK_ACCT,
	A_LOGIN,
#ifdef FEATURE_MODERATE
	A_MODERATE,
#endif /* FEATURE_MODERATE */
	A_OPEN_COMMENTS,
	A_PREFS,
	A_PREFSPOST,
	A_PUBLISH,
	A_REGISTER,
	A_REGISTERPOST,
	A_REPLY,
	A_REPLYPOST,
	A_REPORT,
	A_RSS,
#ifdef FEATURE_SEARCH
	A_SEARCH,
#endif /* FEATURE_SEARCH */
	A_SHOW_DOC,
	A_SUBMISSION,
	A_SUBMISSIONS,
	A_SUBMIT,
	A_SUBMITPOST,
	A_UNBLOCK_ADDR,
	A_UNLOCK_ACCT,
#ifdef FEATURE_VOTE
	A_VOTE
#endif /* FEATURE_VOTE */
};

/* Sets of actions */
#define ACTIONS_RO_COMMON  \
	( SNB(A_ABOUT) \
	| SNB(A_ARCHIVE) \
	| SNB(A_ARTICLE) \
	| SNB(A_CONFIRM) \
	| SNB(A_FRONT) \
	| SNB(A_RSS) \
	| SNB(A_SHOW_DOC) \
	| SNB(A_SUBMISSION) \
	)

#ifdef UNDEADLY_TRANSITION

#ifdef FEATURE_SEARCH
#define ACTIONS_RO ( ACTIONS_RO_COMMON |  SNB(A_ERRATA) | SNB(A_SEARCH) )
#else /* FEATURE_SEARCH */
#define ACTIONS_RO ( ACTIONS_RO_COMMON |  SNB(A_ERRATA) )
#endif /* FEATURE_SEARCH */

#else /* UNDEADLY_TRANSITION */

#ifdef FEATURE_SEARCH
#define ACTIONS_RO ( ACTIONS_RO_COMMON | SNB(A_SEARCH) )
#else /* FEATURE_SEARCH */
#define ACTIONS_RO ( ACTIONS_RO_COMMON )
#endif /* FEATURE_SEARCH */

#endif /* UNDEADLY_TRANSITION */

#define ACTIONS_REGISTER  \
	( SNB(A_REGISTER) \
	| SNB(A_REGISTERPOST) \
	| SNB(A_CONFIRM) \
	)

#define ACTIONS_LOGIN  \
	( SNB(A_LOGIN) \
	| SNB(A_AUTH) \
	)

#define ACTIONS_SUBMIT \
	( SNB(A_SUBMIT) \
	| SNB(A_SUBMITPOST) \
	)

#define ACTIONS_COMMENT \
	( SNB(A_REPLY) \
	| SNB(A_REPLYPOST) \
	)

#define ACTIONS_LOGGED_IN \
	( SNB(A_CHANGEPW) \
	| SNB(A_DEAUTH) \
	| SNB(A_PREFS) \
	| SNB(A_PREFSPOST) \
	)

/* Default rights for members of pre-defined groups */
#define DEFAULT_RIGHTS_ANON  ( ACTIONS_RO | ACTIONS_REGISTER | ACTIONS_LOGIN )
#define DEFAULT_RIGHTS_USER  ( ACTIONS_RO | ACTIONS_SUBMIT | ACTIONS_COMMENT \
    | ACTIONS_LOGGED_IN )


/*
 * Action names (i.e. values for field K_ACTION)
 *
 * If modifying these, remember to maintain the entries in action_table
 * (in fwobac.c) in alphabetical order (for binary searching).
 */
#define AN_ABOUT           "about"
#define AN_ARCHIVE         "archive"
#define AN_ARTICLE         "article"
#define AN_AUTH            "auth"
#define AN_BLOCK_ADDR      "block_addr"
#define AN_CHANGEPW        "changepw"
#define AN_CLOSE_REPORT    "close_report"
#define AN_CLOSE_COMMENTS  "close_comments"
#define AN_CONFIRM         "confirm"
#define AN_DEAUTH          "deauth"
#define AN_DELETE_A        "delete_a"
#define AN_DELETE_C        "delete_c"
#define AN_DELETE_S        "delete_s"
#define AN_EDIT_A          "edit_a"
#define AN_EDIT_S          "edit_s"
#define AN_EDITPOST_A      "editpost_a"
#define AN_EDITPOST_S      "editpost_s"
#ifdef UNDEADLY_TRANSITION
#define AN_ERRATA          "errata"
#endif /* UNDEADLY_TRANSITION */
#define AN_FRONT           "front"
#define AN_LOCK_ACCT       "lock_acct"
#define AN_LOGIN           "login"
#ifdef FEATURE_MODERATE
#define AN_MODERATE        "moderate"
#endif /* FEATURE_MODERATE */
#define AN_OPEN_COMMENTS   "open_comments"
#define AN_PREFS           "prefs"
#define AN_PREFSPOST       "prefspost"
#define AN_PUBLISH         "publish"
#define AN_REGISTER        "register"
#define AN_REGISTERPOST    "registerpost"
#define AN_REPLY           "reply"
#define AN_REPLYPOST       "replypost"
#define AN_REPORT          "report"
#define AN_RSS             "rss"
#ifdef FEATURE_SEARCH
#define AN_SEARCH          "search"
#endif /* FEATURE_SEARCH */
#define AN_SHOW_DOC        "show_doc"
#define AN_SUBMISSION      "submission"
#define AN_SUBMISSIONS     "submissions"
#define AN_SUBMIT          "submit"
#define AN_SUBMITPOST      "submitpost"
#define AN_UNBLOCK_ADDR    "unblock_addr"
#define AN_UNLOCK_ACCT     "unlock_acct"
#ifdef FEATURE_VOTE
#define AN_VOTE            "vote"
#endif /* FEATURE_VOTE */

/* Field values */
#define KV_TYPE_HTML     "html"
#define KV_TYPE_PLAIN    "plain"

/* Configuration entries. */
enum conf_item {
	CI_ARTICLES_DIR,
	CI_CACHE_DIR,
	CI_CAPTCHA_KEY,
	CI_COOKIE_KEY,
	CI_DEFAULT_TOPIC,
	CI_DOMAIN,
	CI_GROUP_RIGHTS,
	CI_HTTPS_PORT,
	CI_REGISTRATION_KEY,
	CI_SUBMISSIONS_DIR,
	CI_TEMPLATE_DIR,
	CI_TEMPLATE_MACRO_CHARS,
#ifdef FEATURE_THEMES
	CI_THEMES,
#endif /* FEATURE_THEMES */
	CI_USER_DB_FILE
};

#define CI_N_ARTICLES_DIR          "articles_dir"
#define CI_N_CACHE_DIR             "cache_dir"
#define CI_N_CAPTCHA_KEY           "captcha_key"
#define CI_N_COOKIE_KEY            "cookie_key"
#define CI_N_DEFAULT_TOPIC         "default_topic"
#define CI_N_DOMAIN                "domain"
#define CI_N_GROUP_RIGHTS          "group_rights"
#define CI_N_HTTPS_PORT            "https_port"
#define CI_N_REGISTRATION_KEY      "registration_key"
#define CI_N_SUBMISSIONS_DIR       "submissions_dir"
#define CI_N_TEMPLATE_DIR          "template_dir"
#define CI_N_TEMPLATE_MACRO_CHARS  "template_macro_chars"
#ifdef FEATURE_THEMES
#define CI_N_THEMES                "themes"
#endif /* FEATURE_THEMES */
#define CI_N_USER_DB_FILE          "user_db_file"

/* Configuration file errors (from callback function) */
#define CPCB_NO_ERR         0
#define CPCB_BAD_NAME_ERR   1
#define CPCB_DUP_VAL_ERR    2
#define CPCB_ALLOC_ERR      3
#define CPCB_INTERNAL_ERR  -1

/* Metadata fields in article/comment files */
#define MD_F_ADDR_AUTHOR    "addr_author"
#define MD_F_ADDR_EDITOR    "addr_editor"
#if defined(COMPAT_LEGACY_R) || defined(COMPAT_LEGACY_W)
#define MD_F_DATE           "date"
#endif /* defined(COMPAT_LEGACY_R) || defined(COMPAT_LEGACY_W) */
#define MD_F_DEPT           "dept"
#define MD_F_EMAIL          "email"
#if defined(COMPAT_LEGACY_R) || defined(COMPAT_LEGACY_W)
#define MD_F_HOST           "host"
#endif /* defined(COMPAT_LEGACY_R) || defined(COMPAT_LEGACY_W) */
#define MD_F_HREF           "href"
#define MD_F_ID_AUTHOR      "id_author"
#define MD_F_ID_EDITOR      "id_editor"
#define MD_F_NAME           "name"
#define MD_F_POLL           "poll"
#define MD_F_PUBDATE        "pubdate"
#define MD_F_SUBJECT        "subject"
#define MD_F_TIME_AUTHORED  "time_authored"
#define MD_F_TIME_EDITED    "time_edited"
#define MD_F_TOPIC          "topic"

/* Formats for metadata fields */
#define MD_FMT     "%s: %s\n"
#define MD_FLAG_FMT  "%s:\n"

/* Pathname components */
#define ARTICLE_FILE    "article"
#define MORE_SUFFIX     ".more"
#define LAST_READ       "lastread"
#define COMMENTS_OPEN   "comments_open"
#define COMMENT_FILE    "comment"
#define COMMENT_MOD     "comment.mod"
#define POLL_FILE       "article.poll"
#define COMMENT_REPORT  "report"
#define RSS_LOG         "rss_log"
#define RSS_SUMMARY     "summary.rss"


/* Path formats relative to template directory */
/* For action A_SHOW_DOC */
#define FMT_TMPL_DOC        "doc/%s.html"
/* For macro MCR_PREFIX_INCLUDE_G */
#define FMT_TMPL_INCLUDE_G  "groups/%s/"
#ifdef FEATURE_THEMES
/* For macro MCR_PREFIX_INCLUDE_G */
#define FMT_TMPL_INCLUDE_T  "themes/%.*s/"
#endif /* FEATURE_THEMES */


/* Names of template files */
#define TMPL_ABOUT               "about.html"
#define TMPL_AI_META_YEAR        "ai_meta_year.html"
#define TMPL_AI_META_MONTH       "ai_meta_month.html"
#define TMPL_AI_META_MONTH_ART   "ai_meta_article.html"
#define TMPL_AI_YEAR_INDEX_DOC   "ai_year_index_doc.html"
#define TMPL_AI_YEAR_LIST_ITEM   "ai_year_list_item.html"
#define TMPL_AL_ARTICLE_ITEM     "al_article_item.html"
#define TMPL_AL_DAY_ITEM         "al_day_item.html"
#define TMPL_AL_EARLIER          "al_earlier.html"
#define TMPL_AL_LATEST           "al_latest.html"
#define TMPL_ARTICLE             "article.html"
#define TMPL_ARTICLE_ITEM        "article_item.html"
#define TMPL_CAPTCHA             "captcha.html"
#ifdef FEATURE_SHOW_THREAD
#define TMPL_COMMENTS_DOC        "comments_doc.html"
#endif /* FEATURE_SHOW_THREAD */
#define TMPL_COMMENT_ITEM        "comment_item.html"
#define TMPL_COMMENT_ITEM_BEGIN  "comment_item_begin.html"
#define TMPL_COMMENT_ITEM_END    "comment_item_end.html"
#define TMPL_COMMENT_LIST_BEGIN  "comment_list_begin.html"
#define TMPL_COMMENT_LIST_END    "comment_list_end.html"
#define TMPL_EDIT_A              "edit_a.html"
#define TMPL_EDIT_PREVIEW        "edit_preview.html"
#define TMPL_EDIT_S              "edit_s.html"
#define TMPL_F_ERR               "form_error.html"
#define TMPL_F_ERR_INVALID       "form_error_invalid.html"
#define TMPL_F_ERR_MISSING       "form_error_missing.html"
#define TMPL_F_ERR_ID_IN_USE     "form_error_id_in_use.html"
#define TMPL_FRONT               "front.html"
#define TMPL_LOGIN               "login.html"
#define TMPL_LOGIN_FAILED        "login_failed.html"
#define TMPL_LOGIN_REG_CONFRMD   "login_reg_confirmed"
#define TMPL_PREFS               "prefs.html"
#define TMPL_REG                 "register.html"
#define TMPL_REG_MAIL            "registration_mail.txt"
#define TMPL_REPLY               "reply.html"
#define TMPL_REPLY_PREVIEW       "reply_preview.html"
#define TMPL_SUBMISSION          "submission.html"
#define TMPL_SUBMISSIONS         "submissions.html"
#define TMPL_SUBMISSION_ITEM     "submission_item.html"
#define TMPL_SUBMIT              "submit.html"
#define TMPL_SUBMIT_PREVIEW      "submit_preview.html"
#define TMPL_SUMMARY_ITEM        "summary_item.rss"
#define TMPL_S_200               "simple_200.html"
#define TMPL_S_200_A_EMAIL_SENT  "simple_200_act_email_sent.html"
#define TMPL_S_200_ALREADY_ACT   "simple_200_already_active.html"
#define TMPL_S_200_LOCKED        "simple_200_locked.html"
#define TMPL_S_200_NYI           "simple_200_nyi.html"
#define TMPL_S_200_PW_CHANGED    "simple_200_pw_changed.html"
#define TMPL_S_200_UNLOCKED      "simple_200_unlocked.html"
#define TMPL_S_200_ASK_DEL_A     "simple_200_ask_del_a.html"
#define TMPL_S_200_ASK_DEL_S     "simple_200_ask_del_s.html"
#ifdef UNDEADLY_TRANSITION
#define TMPL_S_301               "simple_301.html"
#endif /* UNDEADLY_TRANSITION */
#define TMPL_S_302               "simple_302.html"
#define TMPL_S_400               "simple_400.html"
#define TMPL_S_403               "simple_403.html"
#define TMPL_S_403_CMTS_CLOSED   "simple_403_cmts_closed.html"
#define TMPL_S_403_CNFMN_BAD_ID  "simple_403_cnfmn_bad_id.html"
#define TMPL_S_403_CNFMN_INVAL   "simple_403_cnfmn_inval.html"
#define TMPL_S_403_CNFMN_LOCKED  "simple_403_cnfmn_locked.html"
#define TMPL_S_403_HTTPS_REQD    "simple_403_https_reqd.html"
#define TMPL_S_403_INSUF_RIGHTS  "simple_403_insufficient_rights.html"
#define TMPL_S_403_POST_REQD     "simple_403_post_reqd.html"
#define TMPL_S_404               "simple_404.html"
#define TMPL_S_405               "simple_405.html"
#define TMPL_S_500               "simple_500.html"
#define TMPL_S_503               "simple_503.html"

/* Template macros */
#define MCR_PREFIX_COND_A_FOUND  "IF_ARTICLES_FOUND?"
#define MCR_PREFIX_COND_AUTH     "IF_AUTH?"
#define MCR_PREFIX_COND_CMT_REP  "IF_REPORTED?"
#define MCR_PREFIX_COND_CMTS_OP  "IF_CMTS_OPEN?"
#define MCR_PREFIX_COND_DEBUG    "IF_DEBUG?"
#define MCR_PREFIX_COND_DEBUG_B  "IF_DEBUG_BUILD?"
#define MCR_PREFIX_COND_DELAYED  "IF_DELAYED?"
#define MCR_PREFIX_COND_HREF_URL "IF_HREF_IS_URL?"
#define MCR_PREFIX_COND_HTTPS    "IF_HTTPS?"
#define MCR_PREFIX_COND_MORE     "IF_MORE?"
#define MCR_PREFIX_COND_NEW      "IF_NEW?"  /* Implied IF_AUTH? */
#define MCR_PREFIX_COND_NEW_C    "IF_NEW_CMTS?"  /* Implied IF_AUTH? */
#define MCR_PREFIX_COND_NE_NAME  "IF_NON_EMTPY_NAME?"
#define MCR_PREFIX_COND_NE_HREF  "IF_NON_EMTPY_HREF?"
#define MCR_PREFIX_COND_PRV_BODY "IF_PREVIEW_BODY?"
#define MCR_PREFIX_COND_SUBM     "IF_SUBMISSION?"
#define MCR_PREFIX_INCLUDE       "INCLUDE:"
#define MCR_PREFIX_INCLUDE_G     "INCLUDE_BY_GROUP:"
#ifdef FEATURE_THEMES
#define MCR_PREFIX_INCLUDE_T     "INCLUDE_BY_THEME:"
#endif /* FEATURE_THEMES */
#define MCR_PREFIX_TIME_A_FMT    "TIME_A_FMT:"
#define MCR_PREFIX_TIME_E_FMT    "TIME_E_FMT:"
#define MCR_PREFIX_TIME_I_FMT    "TIME_I_FMT:"
#define MCR_PREFIX_TIME_S_FMT    "TIME_S_FMT:"
#define MCR_PREFIX_TIME_FMT      "TIME_FMT:"
#define MCR_PREFIX_SEL_TOPIC     "SELECTED_TOPIC:"
#define MCR_PREFIX_CHCKD_TYPE    "CHECKED_TYPE:"
#define MCR_ACTION               "ACTION"
#define MCR_ADDR_AUTHOR          "ADDR_AUTHOR"
#define MCR_ADDR_EDITOR          "ADDR_EDITOR"
#define MCR_AI_EACH_YEAR         "AI_EACH_YEAR"
#define MCR_AI_MONTH_ARTICLES    "AI_MONTH_ARTICLES"
#define MCR_AI_MONTH_LIST        "AI_MONTH_LIST"
#define MCR_AI_YEAR              "YEAR"
#define MCR_AI_YEAR_INDEX        "AI_YEAR_INDEX"
#define MCR_AL_DAY_ARTICLE_LIST  "AL_DAY_ARTICLE_LIST"
#define MCR_AL_DAY_LIST          "AL_DAY_LIST"
#define MCR_ARTICLE              "ARTICLE"
#define MCR_ARTICLES             "ARTICLES"
#define MCR_AUTHOR_INFO          "AUTHOR_INFO"
#define MCR_BODY                 "BODY"
#define MCR_CAPTCHA              "CAPTCHA"
#define MCR_CAPTCHA_CHK          "CAPTCHA_CHK"
#define MCR_CAPTCHA_TXT          "CAPTCHA_TXT"
#define MCR_CHECKED_POLL         "CHECKED_POLL"
#define MCR_CLIENT               "CLIENT"
#define MCR_COMMENTS             "COMMENTS"
#define MCR_COUNT                "COUNT"
#define MCR_CONTENT0             "CONTENT0"
#define MCR_CONTENT1             "CONTENT1"
#define MCR_DEPT                 "DEPT"
#define MCR_EDITOR_INFO          "EDITOR_INFO"
#define MCR_EMAIL                "EMAIL"
#define MCR_EXTRA                "EXTRA"
#define MCR_FORM_EXTRA           "FORM_X"
#define MCR_F_ERR_INVALID_LIST   "FORM_ERR_INVALID_LIST"
#define MCR_F_ERR_MISSING_LIST   "FORM_ERR_MISSING_LIST"
#define MCR_F_ERR_MSG            "FORM_ERR_MSG"
#define MCR_GROUP                "GROUP"
#define MCR_HMAC                 "HMAC"
#define MCR_HREF                 "HREF"
#define MCR_ID                   "ID"
#define MCR_ID_AUTHOR            "ID_AUTHOR"
#define MCR_ID_EDITOR            "ID_EDITOR"
#define MCR_INDEX                "INDEX"
#define MCR_INTRO                "INTRO"
#define MCR_INTRO_PASSWORD       "INTRO_PASSWORD"
#define MCR_INTRO_PREFS          "INTRO_PREFS"
#define MCR_ITEMS                "ITEMS"
#define MCR_LATEST_LIST          "LATEST_LIST"
#ifdef FEATURE_MODERATE
#define MCR_MODCOUNT             "MODCOUNT"
#define MCR_MODSUM               "MODSUM"
#endif /* FEATURE_MODERATE */
#define MCR_MORE                 "MORE"
#define MCR_NAME                 "NAME"
#ifdef FEATURE_A_PREV_NEXT_LINKS
#define MCR_NAVIGATION           "NAVIGATION"
#endif /* FEATURE_A_PREV_NEXT_LINKS */
#define MCR_OLDER_LIST           "OLDER_LIST"
#define MCR_PID                  "PID"
#define MCR_POLL                 "POLL"
#define MCR_PREVIEW              "PREVIEW"
#define MCR_PUBDATE              "PUBDATE"
#define MCR_PW                   "PW"
#define MCR_PWCONFIRM            "PWCONFIRM"
#define MCR_REG_ADDR             "REG_ADDR"
#define MCR_REG_REF              "REG_REF"
#define MCR_REG_SNOM             "REG_SNOM"
#define MCR_REG_SRVR             "REG_SRVR"
#define MCR_REG_UA               "REG_UA"
#define MCR_REMOTE_ADDR          "REMOTE_ADDR"
#define MCR_SCHEME               "SCHEME"
#define MCR_SCRIPT_NAME          "SCRIPT_NAME"
#define MCR_SERVER_NAME          "SERVER_NAME"
#define MCR_SERVER_PORT          "SERVER_PORT"
#define MCR_SID                  "SID"
#define MCR_STATUS               "STATUS"
#define MCR_SUBJECT              "SUBJECT"
#define MCR_SUBJECT_RAW          "SUBJECT_RAW"
#ifdef FEATURE_THEMES
#define MCR_THEME                "THEME"
#endif /* FEATURE_THEMES */
#ifdef FEATURE_THRES
#define MCR_THRES                "THRES"
#endif /* FEATURE_THRES */
#define MCR_TOPIC                "TOPIC"
#define MCR_URL_EXTRA            "URL_X"
#define MCR_VERSION              "VERSION"

#endif /* !_FWOBAC_H_ */

