/* $Id: html_utils.h,v 1.3 2017/08/05 12:17:41 rlr Exp $ */

/*
 * Copyright (c) 2017 Ross L Richardson <rueda@rlr.id.au>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#ifndef _HTML_UTILS_H_
#define _HTML_UTILS_H_


#include <stdbool.h>
#include <stddef.h>


/*
 * void
 * text_to_html(const char *t, bool nlx, char *h, size_t *sz);
 *
 * Converts from text to simple HTML (or just reports the buffer size required
 * for such a conversion.
 *
 * t must be NULL or a NUL-terminated string.  h must be NULL or a buffer of
 * sufficient size to hold the converted string.  sz must be non-NULL.
 *
 * If t is NULL, *sz is set to zero and, if non-NULL, h is set to the empty
 * string.
 * If t is non-NULL, the type of conversion is determined by nlx:
 * - if nlx is false, only character reference replacement (for '"', '&', '<',
 *   and '>') is undertaken;
 * - if nlx is true, '\r' characters in t are ignored (and elided from
 *   output), leading and trailing '\n' characters are elided, sequences of two
 *   or more '\n' characters delimit paragraphs (<p>/</p>), and single '\n'
 *   characters are converted to linebreaks; if t is the empty string, an
 *   empty paragraph is generated.
 * sz is set to the buffer size required for the converted string.  In if h
 * is non-NULL, the HTML equivalent of t is copied to it.
 */
void
text_to_html(const char *, bool, char *, size_t *);


/*
 * char *
 * alloc_text_to_html(const char *t, bool nlx);
 *
 * Returns a newly allocated buffer containing the text_to_html() conversion
 * of t (according to nlx), or NULL if t is NULL or allocation failure occurs.
 */
char *
alloc_text_to_html(const char *t, bool nlx);


#endif /* !_HTML_UTILS_H_ */

