/* $Id: tmpl.h,v 1.3 2017/08/05 12:26:40 rlr Exp $ */

/*
 * Copyright (c) 2017 Ross Richardson <ross@rlr.id.au>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _TMPL_H_
#define _TMPL_H_


#include <stddef.h>


/* Error codes. */
#define TMPL_NO_ERR        0
#define TMPL_INVALID       1
#define TMPL_INPUT_NOT_WF  2
#define TMPL_EMIT_ERR      3
#define TMPL_REPL_ERR      4
#define TMPL_OPEN_ERR      5
#define TMPL_STAT_ERR      6
#define TMPL_TOO_BIG_ERR   7
#define TMPL_MMAP_ERR      8

/*
 * int
 * tmpl_buf(const void *buf, size_t len, const unsigned char md,
 *     int (*emit)(const void *, size_t, void *), void *ae,
 *     int (*repl)(const void *, size_t, void *), void *ar);
 *
 * tmpl_buf provides a simple templating system for arbitrary data.
 *
 * buf contains arbitrary data of length len.  md is the octet by (pairs of)
 * which "macros" for substitution are identified.  Each macro is contained
 * within a pair of md at each end.  (For example, with md '@', "@@TITLE@@"
 * is a macro named "TITLE".)  A zero-length name is permitted.
 *
 * emit must be non-NULL.  If repl is NULL, emit is called with parameters:
 * buf, len, and ae.  If repl is non-NULL, calls to emit (for macro-free
 * sub-sections of buf, again with ae) and to repl (with macro name, length
 * of macro name, and ar) are interleaved as appropriate.
 *
 * emit and repl should return zero if no error occurs, and non-zero otherwise.
 *
 * Returns TMPL_NO_ERR if no error occurs, or an appropriate error code
 * otherwise.
 */
int
tmpl_buf(const void *, size_t, const unsigned char,
    int (*)(const void *, size_t, void *), void *,
    int (*)(const void *, size_t, void *), void *);


/*
 * int
 * tmpl_file(const char *filepath, const unsigned char md,
 *     int (*emit)(const void *, size_t, void *), void *ae,
 *     int (*repl)(const void *, size_t, void *), void *ar);
 *
 * Wrapper for tmpl_buf() which uses data from file filepath.
 *
 * On OpenBSD, requires pledge(2) promise "rpath".
 */
int
tmpl_file(const char *, const unsigned char,
    int (*)(const void *, size_t, void *), void *,
    int (*)(const void *, size_t, void *), void *);


/*
 * Convenience wrapper for tmpl_buf() which provides the repl() callback
 * with a C string if macro name is a valid C string, or returns TMPL_REPL_ERR
 * otherwise.
 */
int
tmpl_buf_cstr(const void *, size_t, const unsigned char,
    int (*)(const void *, size_t, void *), void *,
    int (*)(const char *, void *), void *);


/*
 * Convenience wrapper for tmpl_file() which provides the repl() callback
 * with a C string if macro name is a valid C string, or returns TMPL_REPL_ERR
 * otherwise.
 */
int
tmpl_file_cstr(const char *, const unsigned char,
    int (*)(const void *, size_t, void *), void *,
    int (*)(const char *, void *), void *);


#endif /* !_TMPL_H_ */

